import { Controller, Inject } from '@nestjs/common';
import { MessagePattern } from '@nestjs/microservices';
import { MailerService } from '../service/mailer.service';
import { Op, QueryTypes } from "sequelize";
import { Cron } from '@nestjs/schedule';

@Controller('mailer')
export class MailerController {

   constructor(
      private readonly mailService: MailerService,
      @Inject('SequelizeInstance') private readonly sequelizeInstance,
   ) { }

   @MessagePattern({ cmd: 'mail' })
   mail(data: any) {

      console.log("controller newer call:", JSON.parse(data));
      //return this.mailService.sendEmail(JSON.parse(data));
      //return this.mail.channelAlgm(JSON.parse(data));
   }

   @MessagePattern({ cmd: 'addproductmail' })
   async Add_Product_Mail(input: any) {
      let data = JSON.parse(input);

      //console.log("addproduct:",data);

      let query = 'select concat("firstName",\' \',"lastName") as name,email from "tblContact" where "contactId"=(select "contactId" from "tblUser" where "userId"=:userId) ';

      let result = await this.sequelizeInstance.query(query, {
         type: QueryTypes.SELECT,
         raw: true,
         replacements: {
            userId: data.userId
         }
      });

      let msg = {
         to: result[0].email,
         from: process.env.EMAIL_FROM,
         templateId: 'd-aa5e57c256a641fdbfbd7486722165e1',
         dynamicTemplateData: {
            name: result[0].name,
         }
      };
      return this.mailService.sendEmail(msg);
   }

   @MessagePattern({ cmd: 'aftersignupmail' })
   async After_Signup_Mail(input: any) {
      let data = JSON.parse(input);

      let msg = {
         to: data.to,
         from: process.env.EMAIL_FROM,
         templateId: 'd-a20e8735754e47edb328e72b9c650772',
         dynamicTemplateData: {
            name: data.name,
         }
      };
      return this.mailService.sendEmail(msg);
   }

   @MessagePattern({ cmd: 'followingvendor' })
   async Buyer_Following_Vendor(input: any) {
      let data = JSON.parse(input);

      let query = 'select "email",concat("firstName",\' \',"lastName") as name from "tblContact" where "channelId"=:channelId and "isActive"=\'1\'  and "deletedAt" is null ';

      let result = await this.sequelizeInstance.query(query, {
         type: QueryTypes.SELECT,
         raw: true,
         replacements: {
            channelId: data.channelId
         }
      });

      result.map(x => {

         let msg = {
            to: x.email,
            from: process.env.EMAIL_FROM,
            templateId: 'd-b3517a78038c43839b0e267f0b62dfa0',
            dynamicTemplateData: {
               name: x.name,
            }
         };
         this.mailService.sendEmail(msg);
      })

      return true;//this.mailService.sendEmail(msg);       
   }

   @MessagePattern({ cmd: 'favoriteproduct' })
   async Buyer_Favorites_Product(input: any) {
      let data = JSON.parse(input);

      let query = 'select "email",concat("firstName",\' \',"lastName") as name from "tblContact" where "channelId"=(select "channelId" from "tblProduct" where "productId"=:productId) and "isActive"=\'1\'  and "deletedAt" is null ';

      let result = await this.sequelizeInstance.query(query, {
         type: QueryTypes.SELECT,
         raw: true,
         replacements: {
            productId: data.productId
         }
      });

      result.map(x => {

         let msg = {
            to: x.email,
            from: process.env.EMAIL_FROM,
            templateId: 'd-5b41e3fb45424532b5957d5e3eb943ce',
            dynamicTemplateData: {
               name: x.name,
            }
         };
         this.mailService.sendEmail(msg);
      })

      return true;//this.mailService.sendEmail(msg);       
   }


   @MessagePattern({ cmd: 'afterapprovalmail' })
   async Channel_Approval_Mail(input: any) {
      let data = JSON.parse(input);

      let query = 'select concat("firstName",\' \',"lastName") as name from "tblContact" where "contactId"=:contactId ';

      let result = await this.sequelizeInstance.query(query, {
         type: QueryTypes.SELECT,
         raw: true,
         replacements: {
            contactId: data.contactId
         }
      });

      let msg = {
         to: data.to,
         from: process.env.EMAIL_FROM,
         templateId: 'd-fadcddce4e6a41e9a73c027c7ca6ed95',
         dynamicTemplateData: {
            name: result[0].name,
            profileurl: data.profileurl
         }
      };
      return this.mailService.sendEmail(msg);
   }

   @MessagePattern({ cmd: 'afterdismissmail' })
   async Channel_Dismiss_Mail(input: any) {
      let data = JSON.parse(input);

      let query = 'select concat("firstName",\' \',"lastName") as name from "tblContact" where "contactId"=:contactId ';

      let result = await this.sequelizeInstance.query(query, {
         type: QueryTypes.SELECT,
         raw: true,
         replacements: {
            contactId: data.contactId
         }
      });

      let msg = {
         to: data.to,
         from: process.env.EMAIL_FROM,
         templateId: 'd-e5136f3dba5044e4b8e87e2b160d61a4',
         dynamicTemplateData: {
            name: result[0].name,
            //profileurl: data.profileurl
         }
      };
      return this.mailService.sendEmail(msg);
   }


   @MessagePattern({ cmd: 'resetpassword' })
   async Reset_Password_Mail(input: any) {
      let data = JSON.parse(input);

      let msg = {
         to: data.email,
         from: process.env.EMAIL_FROM,
         templateId: 'd-bf3f35948d4e4154aa22239acd8b0b66',
         dynamicTemplateData: {
            name: data.name,
            profileurl: data.url
         }
      };
      return this.mailService.sendEmail(msg);
   }

   @MessagePattern({ cmd: 'addusermail' })
   async Add_User_Mail(input: any) {
      let data = JSON.parse(input);

      console.log("data:", data);

      let query = 'select concat("firstName",\' \',"lastName") as name from "tblContact" where "contactId"=:contactId ';

      let result = await this.sequelizeInstance.query(query, {
         type: QueryTypes.SELECT,
         raw: true,
         replacements: {
            contactId: data.accountOwner
         }
      });

      let msg = {
         to: data.to,
         from: process.env.EMAIL_FROM,
         templateId: 'd-959d91d142794da6b90e11b37933d4ec',
         dynamicTemplateData: {
            name: data.name,
            profileurl: data.profileurl,
            userId: data.userId,
            password: data.password,
            accountOwner: result[0].name
         }
      };
      return this.mailService.sendEmail(msg);
   }

   //Time Based mail
   @MessagePattern({ cmd: 'neverconnect' })
   async Buyer_Nerver_Connect(input: any) {
      let data = JSON.parse(input);

      let query = 'select (select date("lastLogin") from "tblUser" where "contactId"=tc."contactId") as lastLogin,email,concat("firstName",\' \',"lastName") as name from "tblContact" as tc '
         + ' where "channelId" in(select "channelId" from "tblUser" where date(now())-date("lastLogin")>15 and '
         + ' "channelId" in(select "channelId" from "tblChannel"  where "channelTypeId" in(3,4,5) and "isActive"=\'1\' and "deletedAt" is null)) and "isActive"=\'1\'  and "deletedAt" is null ';

      let result = await this.sequelizeInstance.query(query, {
         type: QueryTypes.SELECT,
         raw: true,
         replacements: {
            productId: data.productId
         }
      }).then(res => {
         return res.filter(x => x.lastlogin != null);
      })

      for (let i = 0; i < result.length; i = i + 10) {
         result.slice(i, 10);

         result.map(x => {

            let msg = {
               to: x.email,
               from: process.env.EMAIL_FROM,
               templateId: 'd-acbf35968148481496d4182fe02b0518',
               dynamicTemplateData: {
                  name: x.name,
                  lastLogin: x.lastlogin,
                  account_url: 'http://dev-fo.channelhub.me/'
               }
            };
            this.mailService.sendEmail(msg);
         })
      }

      return true;//this.mailService.sendEmail(msg);       
   }

   //@MessagePattern({ cmd: 'buyersuggestionmail' })
   @Cron('00 01 * * *')
   async Buyer_Suggestion() {

      let cquery = 'select "contactId","suggestion_comm",(select concat("firstName",\' \',"lastName") from "tblContact" where "contactId"=tu."contactId")as name,(select "email" from "tblContact" where "contactId"=tu."contactId"), '
         + '(select date(now())-max(date("createdAt")) from "tblChannelMatching" where "request_to"=tu."contactId") as "period" from "tblUser" as tu where  "deletedAt" is null and "isActive"=\'1\' '
         //+ 'and "contactId" in(select "requested_to" from "tblChannelMatching" where "deletedAt" is null) 
         + 'and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId"=2 and "isActive"=\'1\' and "deletedAt" is null and "signUpStatusId"=15) limit 10 ';

      let channels = await this.sequelizeInstance.query(cquery, {
         type: QueryTypes.SELECT,
         raw: true
      });

      if (channels.length > 0) {

         channels.map(async (ch) => {
            let daysDiff;
            if (ch.suggestion_comm == 1)
               daysDiff = 7;
            else if (ch.suggestion_comm == 2)
               daysDiff = 14;
            else if (ch.suggestion_comm == 3)
               daysDiff = 30;


            if (ch.period == daysDiff || ch.period == 0 || ch.period == null) {
               let query = 'select "channelId","partnerChannelId","requested_to" as request_to,"requested_by" as request_by from "tblChannelJCT" as jct where "connectionStatusTypeId"=1 and "deletedat" is null and "requested_to"=:contactId '
                  //+ 'and "partnerChannelId" in(select "channelId" from "tblChannel" where "channelTypeId"=2) '
                  + 'and "requested_by" not in(select "request_by" from "tblChannelMatching" as cm where cm."request_to"=:contactId '
                  + 'and "mailStatus"=\'1\') limit 5 ';

               let suggestion = await this.sequelizeInstance.query(query, {
                  type: QueryTypes.SELECT,
                  raw: true,
                  replacements: {
                     contactId: ch.contactId
                  }
               });

               //console.log("buyerData:", suggestion);

               if (suggestion.length > 0) {
                  let bquery = 'select "companyName",concat(\'' + process.env.DOMAIN + '/admin/channel/channelDetail/\',"channelId") as "companyUrl",case when "channelTypeId"=3 then \'Distributor\' when "channelTypeId"=4 then \'Retailer\' when "channelTypeId"=5 then \'Reseller\' end as "companyType", '
                     + '(select "country" from "tblCountry" where "countryId"=tc."countryId"), '
                     + '(select regexp_replace("detailDesc", E\'<[^>]+>\', \'\', \'gi\') from "tblChannelDetail" where "channelId"=tc."channelId") as tweetpitch,'
                     + '(select concat(\'' + process.env.REACT_APP_CDN_URL + '\',"documentPath") from "tblDocument" where "documentId" in (select "channelLogoId" from "tblChannelDetail" where "channelId"=tc."channelId")) as "companyLogo" '
                     + 'from "tblChannel" as tc where "channelId" in(:buyer) ';

                  let buyerData = await this.sequelizeInstance.query(bquery, {
                     type: QueryTypes.SELECT,
                     raw: true,
                     replacements: {
                        buyer: suggestion?.map(x => x.channelId),
                     }
                  });

                  //console.log("buyerData:", buyerData);


                  let msg = {
                     to: ch.email,// 'nagakin97@gmail.com',
                     from: process.env.EMAIL_FROM,
                     templateId: 'd-415bbdef638b4531834791d3def9c7ab',
                     dynamicTemplateData: {
                        name: ch.name,
                        buyer: buyerData
                     }
                  };
                  this.mailService.sendChannelSuggestion(msg, suggestion);
               }
            }


         });


      }

      return true;//this.mailService.sendEmail(msg);       
   }

   @Cron('50 12 * * *')
   async Products_Suggestion() {

      let cquery = 'select "channelId","contactId","suggestion_comm",(select concat("firstName",\' \',"lastName") from "tblContact" where "contactId"=tu."contactId")as name,(select "email" from "tblContact" where "contactId"=tu."contactId"), '
         + '(select date(now())-max(date("createdAt")) from "tblProductMatching" where "contactId"=tu."contactId") as "period" from "tblUser" as tu where  "deletedAt" is null and "isActive"=\'1\' '
         //+ 'and "contactId" in(select "requested_to" from "tblChannelMatching" where "deletedAt" is null) 
         + 'and "channelId" in(select "channelId" from "tblChannel" where "channelTypeId" in(3,4,5) and "isActive"=\'1\' and "deletedAt" is null and "signUpStatusId"=15) limit 10 ';

      let channels = await this.sequelizeInstance.query(cquery, {
         type: QueryTypes.SELECT,
         raw: true
      });

      //console.log("channels", channels.length);

      if (channels.length > 0) {


         channels.map(async (ch) => {

            let suggestion = [];
            let daysDiff;
            if (ch.suggestion_comm == 1)
               daysDiff = 7;
            else if (ch.suggestion_comm == 2)
               daysDiff = 14;
            else if (ch.suggestion_comm == 3)
               daysDiff = 30;


            if (ch.period == daysDiff || ch.period == 0 || ch.period == null) {

               let bquery = 'select pc."productCategorie",concat(tcon."firstName",\' \',tcon."lastName") as name,tcon."email" from "tblInterestedProductCategory" as ipc inner join "tblProductCategories" as pc '
                  + 'on pc."productCategorieId"=ipc."productCategorieId" inner join "tblContact" as tcon on tcon."contactId"=ipc."contactId" '
                  + 'where ipc."contactId"=:contactId and ipc."deletedAt" is null';

               let productCat = await this.sequelizeInstance.query(bquery, {
                  type: QueryTypes.SELECT,
                  raw: true,
                  replacements: {
                     contactId: ch.contactId,
                  }
               });

               let name, email, matchedProducts, more = 1;
               productCat?.map(x => {
                  name = x.name;
                  email = x.email;
                  if (more <= 4 && more > 1)
                     matchedProducts = matchedProducts + " - " + x.productCategorie;
                  if (more == 1)
                     matchedProducts = x.productCategorie;

                  more = more + 1;
               })

               let query = 'select tp."productId",tp."product" as "productName",tp."productSKU",(select "companyName" from "tblChannel" where "channelId"="partnerChannelId"), '
                  + '(select "country" from "tblCountry" where "countryId" in(select "countryId" from "tblChannel" where "channelId"="partnerChannelId")), '
                  + '(select regexp_replace("detailDesc", E\'<[^>]+>\', \'\', \'gi\') as "tweetPitch" from "tblChannelDetail" where "channelId"="partnerChannelId"), '
                  + '(select "documentPath" from "tblDocument" where "documentId"=tp."defaultimageid") as "productImage", '
                  + '(select "retailerprice" from "tblproductpricedetail" where "productId"=tp."productId") as price, '
                  + '(select sum("existproductrating")/count(*) from "tblproductexistrating" where "productId"=tp."productId") as rating '
                  + 'from "tblProductJCT" as pjct inner join "tblProduct" as tp on tp."productId"=pjct."productId" and tp."productId" not in(select "productId" from "tblProductMatching" as pm where pm."contactId"=:contactId and pm."mailStatus"=\'1\') '
                  + 'where "requested_by"=:contactId limit 3';

               let product = await this.sequelizeInstance.query(query, {
                  type: QueryTypes.SELECT,
                  raw: true,
                  replacements: {
                     //channelId:data.channelId,
                     contactId: ch.contactId
                  }
               });

               if (product && product.length > 0) {
                  product.map(x => {
                     Object.assign(x, {
                        productUrl: process.env.DOMAIN + "/productDetailView/" + x.productSKU,
                        keyRetailer: "Amazon, Flipkart, Acuchen, Carrefour"
                     });
                     if (x.rating) {
                        let rat = "";
                        for (let i = 0; i < x.rating; i++)
                           rat += "*";

                        x.rating = rat;
                     }
                     x.productImage = process.env.REACT_APP_CDN_URL + x.productImage;
                     let obj = {
                        channelId: +ch.channelId,
                        contactId: +ch.contactId,
                        productId: +x.productId
                     }
                     suggestion.push(obj);
                  })

                  let result = {
                     name: name,
                     matchedProducts: matchedProducts,
                     more: more - 1 > 4 ? more - 1 : 0,
                     product: product
                  }

                  //console.log("result:",result,email);

                  let msg = {
                     to: email,//'nagakin97@gmail.com',
                     from: process.env.EMAIL_FROM,
                     templateId: 'd-452c8584e1584a06ba59e8cce363cb51',
                     dynamicTemplateData: result
                  };

                  await this.mailService.sendProductSuggestion(msg, suggestion);
               }

            }


         });


      }

      return true;//this.mailService.sendEmail(msg);       
   }


   @MessagePattern({ cmd: 'productsuggestionmail' })
   async Product_Suggestion(input: any) {
      let data = JSON.parse(input);
      console.log("data:", data);

      let bquery = 'select pc."productCategorie",concat(tcon."firstName",\' \',tcon."lastName") as name,tcon."email" from "tblInterestedProductCategory" as ipc inner join "tblProductCategories" as pc '
         + 'on pc."productCategorieId"=ipc."productCategorieId" inner join "tblContact" as tcon on tcon."contactId"=ipc."contactId" '
         + 'where ipc."contactId"=:contactId and ipc."deletedAt" is null';

      let productCat = await this.sequelizeInstance.query(bquery, {
         type: QueryTypes.SELECT,
         raw: true,
         replacements: {
            contactId: data.contactId,
         }
      });

      let name, email, matchedProducts, more = 1;
      productCat.map(x => {
         name = x.name;
         email = x.email;
         if (more <= 4 && more > 1)
            matchedProducts = matchedProducts + " - " + x.productCategorie;
         if (more == 1)
            matchedProducts = x.productCategorie;

         more = more + 1;
      })

      let query = 'select tp."product" as "productName",tp."productSKU",(select "companyName" from "tblChannel" where "channelId"="partnerChannelId"), '
         + '(select "country" from "tblCountry" where "countryId" in(select "countryId" from "tblChannel" where "channelId"="partnerChannelId")), '
         + '(select regexp_replace("detailDesc", E\'<[^>]+>\', \'\', \'gi\') as "tweetPitch" from "tblChannelDetail" where "channelId"="partnerChannelId"), '
         + '(select "documentPath" from "tblDocument" where "documentId"=tp."defaultimageid") as "productImage", '
         + '(select "retailerprice" from "tblproductpricedetail" where "productId"=tp."productId") as price, '
         + '(select sum("existproductrating")/count(*) from "tblproductexistrating" where "productId"=tp."productId") as rating '
         + 'from "tblProductJCT" as pjct inner join "tblProduct" as tp on tp."productId"=pjct."productId" '
         + 'where "requested_by"=:contactId limit 3';

      let product = await this.sequelizeInstance.query(query, {
         type: QueryTypes.SELECT,
         raw: true,
         replacements: {
            //channelId:data.channelId,
            contactId: data.contactId
         }
      });

      if (product && product.length > 0) {
         product.map(x => {
            Object.assign(x, {
               productUrl: process.env.DOMAIN + "/productDetailView/" + x.productSKU,
               keyRetailer: "Amazon, Flipkart, Acuchen, Carrefour"
            });
            if (x.rating) {
               let rat = "";
               for (let i = 0; i < x.rating; i++)
                  rat += "*";

               x.rating = rat;
            }

            x.productImage = process.env.REACT_APP_CDN_URL + x.productImage;
         })

         let result = {
            name: name,
            matchedProducts: matchedProducts,
            more: more - 1 > 4 ? more - 1 : 0,
            product: product
         }

         //console.log("result:",result,email);

         let msg = {
            to: email,
            from: process.env.EMAIL_FROM,
            templateId: 'd-452c8584e1584a06ba59e8cce363cb51',
            dynamicTemplateData: result
         };
         this.mailService.sendEmail(msg);
      }

      return true;
   }

   //Guest Product Enquiry
   @MessagePattern({ cmd: 'guestproductenquiry' })
   async guest_product_enquiry(input: any) {
      let data = JSON.parse(input);

      let query = 'select "product" from "tblProduct" where "productId"=:productId';

      let result = await this.sequelizeInstance.query(query, {
         type: QueryTypes.SELECT,
         raw: true,
         replacements: {
            productId: data.productId
         }
      }).then(res => {
         return res[0];
      });

      let msg = {
         to: data.email,
         from: process.env.EMAIL_FROM,
         templateId: 'd-61f83a7b6627419cbbe8361f6188e343',
         dynamicTemplateData: {
            name: data.email.split('@')[0],
            productname: result.product
         }
      }

      this.mailService.sendEmail(msg);

      return true;
   }

   @MessagePattern({ cmd: 'connectiondismissmail' })
   async Connection_Dismiss_Mail(input: any) {
      let data = JSON.parse(input);
      //      console.log("addproduct:", data);      

      let query = 'select "companyName",(select concat(\'' + process.env.REACT_APP_CDN_URL + '\',"documentPath") from "tblDocument" where "documentId"=cd."channelLogoId")as "companyLogo",concat("firstName",\' \',"lastName")as "contactName","jobTitle",(select "country" from "tblCountry" where "countryId"=tc."countryId") '
         + 'from "tblContact" as tc inner join "tblChannel" as ch on ch."channelId"=tc."channelId" inner join "tblChannelDetail" as cd on cd."channelId"=tc."channelId" where "contactId"=:req_contactId ';


      let result = await this.sequelizeInstance.query(query, {
         type: QueryTypes.SELECT,
         raw: true,
         replacements: {
            req_contactId: data.req_contact
            //req_channelId: data.req_channel
         }
      });

      let templateData = result[0];
      Object.assign(templateData, {
         receiver_name: data.receiver_name,
         companyType: 'VENDOR'
      });

      //console.log("conn:", result[0]);

      let msg = {
         to: data.receiver_mail,
         from: process.env.EMAIL_FROM,
         templateId: 'd-1313b0cf495d449284e98e982ebc3c73',
         dynamicTemplateData: templateData
      };
      return this.mailService.sendEmail(msg);
   }

   @MessagePattern({ cmd: 'connectionapprovemail' })
   async Connection_Approve_Mail(input: any) {
      let data = JSON.parse(input);

      // let companyType;
      // if (data.typeId == 2)
      //    companyType = 'VENDOR'
      // else if (data.typeId == 3)
      //    companyType = 'DISTRIBUTOR'
      //    else if (data.typeId == 4)
      //    companyType = 'RETAILER'
      //    else if (data.typeId == 3)
      //    companyType = 'RESELLER'
      //console.log("addproduct:", data);
      let query = 'select concat(\'' + process.env.DOMAIN + '/admin/channel/channelDetail/\',tc."channelId") as "companyUrl","companyName",(select concat(\'' + process.env.REACT_APP_CDN_URL + '\',"documentPath") from "tblDocument" where "documentId"=tc."documentId")as "profile_url",concat("firstName",\' \',"lastName")as "contactName","jobTitle",(select "country" from "tblCountry" where "countryId"=tc."countryId") '
         + 'from "tblContact" as tc inner join "tblChannel" as ch on ch."channelId"=tc."channelId"  where "contactId"=:req_contactId ';

      let result = await this.sequelizeInstance.query(query, {
         type: QueryTypes.SELECT,
         raw: true,
         replacements: {
            req_contactId: data.req_contact
            //req_channelId: data.req_channel
         }
      });

      let templateData = result[0];
      Object.assign(templateData, {
         receiver_name: data.receiver_name,
         companyType: 'VENDOR'
         // receiver_photo: data.receiver_photo, connect_url: process.env.DOMAIN + '/admin/mynetwork'
      });
      //console.log("conn:", result[0]);
      let msg = {
         to: data.receiver_mail,
         from: process.env.EMAIL_FROM,
         templateId: 'd-3a0481920ac04363a7be12a935ed20c2',
         dynamicTemplateData: templateData
      };
      return this.mailService.sendEmail(msg);
   }

   @MessagePattern({ cmd: 'connectionrequestmail' })
   async Connection_Request_Mail(input: any) {
      let data = JSON.parse(input);

      //      console.log("addproduct:", data);

      let query; let tempId;
      if (data.typeId == 3) {
         tempId = 'd-68e4ffef524849958a0ac8a55ee4381d';
         query = 'select concat(\'' + process.env.DOMAIN + '/admin/channel/channelDetail/\',cd."channelId") as "companyUrl","companyName",regexp_replace("detailDesc", E\'<[^>]+>\', \'\', \'gi\') as detailDesc,(select concat(\'' + process.env.REACT_APP_CDN_URL + '\',"documentPath") from "tblDocument" where "documentId"=cd."channelLogoId")as "companyLogo",concat("firstName",\' \',"lastName")as name,"jobTitle",(select "country" from "tblCountry" where "countryId"=tc."countryId"), '
            + '(select array_to_json(array_agg(row_to_json(br))) from (select tb."brandName" as item_name,concat(\'' + process.env.REACT_APP_CDN_URL + '\',td."documentPath") as image_Url from "tblBrand" tb inner join "tblDocument" td on td."documentId"=tb."documentId" where "id" in(select "brandId" from "tblChannelBrand" where "channelId"=:req_channelId and "deletedAt" is null))br) as brand,'
            + '(select array_to_json(array_agg(row_to_json(x))) from (Select pc."productCategorie" as item_name from "tblChannelProductCategorie" cpc inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" where cpc."channelId"=:req_channelId  and cpc."deletedAt" is null)x) as company_cat, '
            + '(select array_to_json(array_agg(row_to_json(z))) from (Select pt."profileType" as item_name from "tblProfileType" pt where "profileTypeId" in (select "profileTypeId" from "tblChannelCustomerProfile" cp where cp."channelId"=:req_channelId and cp."deletedAt" is null))z) as retail_customer_profile '
            + 'from "tblContact" as tc inner join "tblChannel" as ch on ch."channelId"=tc."channelId" inner join "tblChannelDetail" as cd on cd."channelId"=tc."channelId" where "contactId"=:req_contactId ';
      }
      else if (data.typeId == 4 || data.typeId == 5) {
         tempId = 'd-d565f18e12514d4c9491cdad31c43116';
         query = 'select concat(\'' + process.env.DOMAIN + '/admin/channel/channelDetail/\',cd."channelId") as "companyUrl","companyName",regexp_replace("detailDesc", E\'<[^>]+>\', \'\', \'gi\') as detailDesc,(select concat(\'' + process.env.REACT_APP_CDN_URL + '\',"documentPath") from "tblDocument" where "documentId"=cd."channelLogoId")as "companyLogo",concat("firstName",\' \',"lastName")as name,"jobTitle",(select "country" from "tblCountry" where "countryId"=tc."countryId"), '
            //+ '(select array_to_json(array_agg(row_to_json(br))) from (select tb."brandName" as item_name,concat(\'' + process.env.REACT_APP_CDN_URL + '\',td."documentPath") as image_Url from "tblBrand" tb inner join "tblDocument" td on td."documentId"=tb."documentId" where "id" in(select "brandId" from "tblChannelBrand" where "channelId"=:req_channelId and "deletedAt" is null))br) as brand,'
            + '(select array_to_json(array_agg(row_to_json(x))) from (Select pc."productCategorie" as item_name from "tblChannelProductCategorie" cpc inner join "tblProductCategories" pc on pc."productCategorieId" = cpc."productCategorieId" where cpc."channelId"=:req_channelId  and cpc."deletedAt" is null)x) as company_cat, '
            //+ '(select array_to_json(array_agg(row_to_json(z))) from (Select pt."profileType" as item_name from "tblProfileType" pt where "profileTypeId" in (select "profileTypeId" from "tblChannelCustomerProfile" cp where cp."channelId"=:req_channelId and cp."deletedAt" is null))z) as retail_customer_profile '
            + 'from "tblContact" as tc inner join "tblChannel" as ch on ch."channelId"=tc."channelId" inner join "tblChannelDetail" as cd on cd."channelId"=tc."channelId" where "contactId"=:req_contactId ';
      }

      let result = await this.sequelizeInstance.query(query, {
         type: QueryTypes.SELECT,
         raw: true,
         replacements: {
            req_contactId: data.req_contact,
            req_channelId: data.req_channel
         }
      });

      let templateData = result[0];
      Object.assign(templateData, {
         message: data.note, receiver_name: data.receiver_name,
         receiver_photo: data.receiver_photo, connect_url: process.env.DOMAIN + '/admin/mynetwork'
      });

      //console.log("conn:", result[0]);

      let msg = {
         to: data.receiver_mail,
         from: process.env.EMAIL_FROM,
         templateId: tempId,
         dynamicTemplateData: templateData
      };
      return this.mailService.sendEmail(msg);
   }

}
